@@ -1,4 +1,5 @@
 from keyword import iskeyword
+import re
 import sphinxify
 import typing
 import yaml
@@ -61,7 +62,7 @@ def _using_signature(fn):
 
 class Hooks:
     """
-        Header2Whatever hooks used for generating C++ wrappers
+    Header2Whatever hooks used for generating C++ wrappers
     """
 
     _qualname_bad = ":<>="
@@ -97,18 +98,25 @@ class Hooks:
             v["x_module_var"] = "m"
 
     def _get_type_caster_includes(self):
+        seps = re.compile(r"[<>\(\)]")
         includes = set()
         for typename in self.types:
             tmpl_idx = typename.find("<")
-            if tmpl_idx != -1:
-                typename = typename[:tmpl_idx]
+            if tmpl_idx == -1:
+                typenames = [typename]
+            else:
+                typenames = [typename[:tmpl_idx]] + seps.split(
+                    typename[tmpl_idx:].replace(" ", "")
+                )
 
-            header = self.casters.get(typename)
-            if header:
-                includes.add(header)
+            for typename in typenames:
+                if typename:
+                    header = self.casters.get(typename)
+                    if header:
+                        includes.add(header)
         return sorted(includes)
 
-    def _set_name(self, name, data, strip_prefixes=None):
+    def _set_name(self, name, data, strip_prefixes=None, is_operator=False):
         if data.rename:
             return data.rename
 
@@ -119,14 +127,14 @@ class Hooks:
             for pfx in strip_prefixes:
                 if name.startswith(pfx):
                     n = name[len(pfx) :]
-                    if not n.isnumeric():
+                    if n.isidentifier():
                         name = n
                         break
 
         if iskeyword(name):
             return f"{name}_"
-        elif name.isnumeric():
-            raise ValueError(f"name cannot be all numbers ({name})")
+        if not name.isidentifier() and not is_operator:
+            raise ValueError(f"name {name!r} is not a valid identifier")
 
         return name
 
@@ -230,9 +238,10 @@ class Hooks:
             qualname = tmpl_data.qualname
             if "::" not in qualname:
                 qualname = f"::{qualname}"
-            tmpl_data = tmpl_data.dict()
-            tmpl_data["x_qualname_"] = qualname.translate(self._qualname_trans)
-            templates[k] = tmpl_data
+            tmpl_data_d = tmpl_data.dict()
+            tmpl_data_d["x_qualname_"] = qualname.translate(self._qualname_trans)
+            self._add_subpackage(tmpl_data_d, tmpl_data)
+            templates[k] = tmpl_data_d
 
         data["templates"] = templates
 
@@ -240,7 +249,7 @@ class Hooks:
         """shared with methods/functions"""
 
         # Python exposed function name converted to camelcase
-        x_name = self._set_name(fn["name"], data)
+        x_name = self._set_name(fn["name"], data, is_operator=fn.get("operator", False))
         if not data.rename and not x_name[:2].isupper():
             x_name = x_name[0].lower() + x_name[1:]
 
@@ -597,6 +606,11 @@ class Hooks:
         template_parameter_list = ""
 
         if class_data.template_params:
+            if class_data.subpackage:
+                raise ValueError(
+                    f"{cls_name}: classes with subpackages must define subpackage on template instantiation"
+                )
+
             template_args = []
             template_params = []
 
